import os
import fnmatch


def parse_gitignore(
    gitignore_path=".gitignore", ignored_files=[".git/", ".venv/", ".github/"]
):
    """
    解析 .gitignore 文件，返回忽略规则列表
    """

    if os.path.exists(gitignore_path):
        with open(gitignore_path, "r") as gitignore_file:
            for line in gitignore_file:
                line = line.strip()
                if line and not line.startswith("#"):
                    ignored_files.append(line)
    return ignored_files


def is_gitignored(file_path, gitignore_rules):
    """
    Check if a file path is ignored according to the given gitignore rules.

    Args:
        file_path (str): The absolute path to the file.
        gitignore_rules (list): List of gitignore rules.

    Returns:
        bool: True if the file is ignored, False otherwise.
    """
    for rule in gitignore_rules:
        # Handle comments and empty lines
        if not rule or rule.startswith("#"):
            continue
        # Handle negated rules
        negated = False
        if rule.startswith("!"):
            negated = True
            rule = rule[1:]

        # Check if the file matches the rule
        if fnmatch.fnmatch(file_path, rule):
            return not negated
        if rule in str(file_path):
            return not negated

    return False


def get_files_in_directory(directory, should_ignore=True, ignore_file_path=None):
    """
    获取指定目录下所有文件，排除.gitignore中列出的文件
    """

    file_list = []
    ignore_file_path = (
        ignore_file_path
        if ignore_file_path is not None
        else os.path.join(".", ".gitignore")
    )
    gitignore_rules = parse_gitignore(ignore_file_path)
    print(ignore_file_path)

    for root, dirs, files in os.walk(directory):
        for file in files:
            file_path = os.path.join(root, file)
            # 检查文件路径是否匹配.gitignore中的规则
            if should_ignore:
                if not is_gitignored(file_path, gitignore_rules):
                    file_list.append(file_path)
            else:
                file_list.append(file_path)
    return file_list


# Example usage 获取当前所有排出掉 gitignore 中的文件
# current_directory = "."  # 当前目录
# files = get_files_in_directory(current_directory)
# print(files)



from way3 import get_current_dir, get_files_in_directory

import unittest


class TestFileFind(unittest.TestCase):
    def test_result(self):
        result = get_current_dir(__file__)
        self.assertIn("way3/tests", result, "result fail: expected file")

    def test_get_files_in_directory(self):
        current_directory = "tests"  # 当前目录
        files = get_files_in_directory(current_directory)
        self.assertIn(
            "tests/test_file_find.py", files, "result fail: expected test_file_find.py"
        )
        self.assertEqual(len(files) > 1, True, "result fail: expected > 3")



name = "way3"

# File OP
from .file_op.create_file import create_file as create_file  # noqa: E402
from .file_op.create_file import add_to_file_end as add_to_file_end  # noqa: E402
from .file_op.create_file import delete_file as delete_file  # noqa: E402


# File Find
from .file_find.current_dir import get_current_dir as get_current_dir  # noqa: E402
from .file_find.traverse_files_from_folder import parse_gitignore as parse_gitignore  # noqa: E402
from .file_find.traverse_files_from_folder import is_gitignored as is_gitignored  # noqa: E402
from .file_find.traverse_files_from_folder import (  # noqa: E402
    get_files_in_directory as get_files_in_directory,
)



import os
import logging
from typing import Union


def create_file(
    file_name: str,
    content: str,
    file_path: Union[str, os.PathLike] = ".",
    force_rewrite: bool = True,
) -> tuple[bool, str]:
    if file_name.startswith(os.sep):
        file_path = file_name
    else:
        file_path = os.path.join(file_path, file_name)

    if not force_rewrite and os.path.exists(file_path):
        logging.warning("File already exists: " + file_path)
        return False, f"Failure, File already exists: {file_path}"

    with open(file_path, "w") as file:
        file.write(content)

    return True, os.path.abspath(file_path)


def add_to_file_end(
    file_name: str,
    content: str,
    file_path: Union[str, os.PathLike] = ".",
    create_if_not_exist: bool = True,
) -> tuple[bool, str]:
    if file_name.startswith(os.sep):
        file_path = file_name
    else:
        file_path = os.path.join(file_path, file_name)

    if not create_if_not_exist and not os.path.exists(file_path):
        logging.warning("File does not exist: " + file_path)
        return False, f"Failure, File does not exist: {file_path}"

    with open(file_path, "a") as file:
        file.write(content)

    return True, {os.path.abspath(file_path)}


def delete_file(
    file_name: str, file_path: Union[str, os.PathLike] = "."
) -> tuple[bool, str]:
    if file_name.startswith(os.sep):
        file_path = file_name
    else:
        file_path = os.path.join(file_path, file_name)

    if os.path.exists(file_path):
        os.remove(file_path)
        return True, {os.path.abspath(file_path)}
    else:
        logging.warning("File does not exist: " + file_path)
        return False, f"Failure, File does not exist: {file_path}"



from way3 import create_file, delete_file

import unittest
import os


class TestFileOp(unittest.TestCase):
    def test_create_file(self):
        result = create_file("./test.txt", "test")
        with open("./test.txt", "r") as f:
            file_content = f.read()
        self.assertEqual(result[0], True, "result fail: expected file")
        self.assertEqual(file_content, "test", "result fail: expected file")
        if os.path.isfile("test.txt"):
            os.remove("test.txt")
            print("test.txt file deleted.")

    def test_delete_file(self):
        result = create_file("./test.txt", "test")
        self.assertEqual(result[0], True, "result fail: expected file")
        delete_file("./test.txt")
        self.assertEqual(
            os.path.isfile("test.txt"), False, "result fail: expected False"
        )
